package riciclaggio_con_liste;

import java.util.*;

public class Riciclaggio {
	
	private static final int MAX_MATERIALI = 200;
		
	private Materiale[] materiali;
	private int numMateriali; // contiene il numero di materiali correntemente nel vettore
	private List<Punto> punti; 
	
	public Riciclaggio(Materiale[] materiali, int numM, List<Punto> punti) {
		this.materiali = materiali;
		this.numMateriali = numM;
		this.punti = punti;
	}
	
	public Riciclaggio() {
		this(new Materiale[MAX_MATERIALI], 0, new ArrayList<Punto>());
	}
	
	// punto 1)
	public boolean inserisciMateriale(Materiale m){
		boolean inserito=false;
		if(numMateriali < MAX_MATERIALI){
			materiali[numMateriali++]=m;
			inserito = true;
		}
		return inserito;
	}
	
	// punto 2)
	public Materiale cercaMateriale(int codMateriale) {
		Materiale materiale = null;
		boolean trovato = false;
		for(int i=0; i < numMateriali && !trovato; i++)
			if(materiali[i].getCodice() == codMateriale) {
				trovato = true;
				materiale = materiali[i];
			}
		return materiale;
	}
	
	
	// punto 3)
	public int materialiEconomiciPerTipo(String tipo, double costoMassimo){
		int numStampati=0;
		
		if(tipo.equalsIgnoreCase("biologico"))
			for(int i=0; i<numMateriali; i++) {
				if(materiali[i] instanceof MaterialeBiologico && materiali[i].getCostoRicicloAlKg() < costoMassimo){
					System.out.println(materiali[i]); // chiamata implicita al metodo toString()
					numStampati++;
				}
			}
		else if(tipo.equalsIgnoreCase("sintetico"))
			for(int i=0; i<numMateriali; i++) {
				if(materiali[i] instanceof MaterialeSintetico && materiali[i].getCostoRicicloAlKg() < costoMassimo){
					System.out.println(materiali[i]); // chiamata implicita al metodo toString()
					numStampati++;
				}
			}
		else if(tipo.equalsIgnoreCase("metallo"))
			for(int i=0; i<numMateriali; i++) {
				if(materiali[i] instanceof MaterialeMetallo && materiali[i].getCostoRicicloAlKg() < costoMassimo){
					System.out.println(materiali[i]); // chiamata implicita al metodo toString()
					numStampati++;
				}		
			} 
		else 
			System.out.println("Errore: tipo di materiale non differenziabile!");
		return(numStampati);
	}
	
	/* 
	 * VERSIONE ALTERNATIVA punto 3)
	 *
	public int materialiEconomiciPerTipo(String tipo, double costoMassimo){
		int numStampati=0;
		
		for(int i=0; i<numMateriali; i++){
			if(materiali[i].getCostoRicicloAlKg() < costoMassimo){
				if(tipo.equalsIgnoreCase("biologico")){
					if(materiali[i] instanceof MaterialeBiologico){
						System.out.println(materiali[i]); // chiamata implicita al metodo toString()
						numStampati++;
					}
				}
				else if(tipo.equalsIgnoreCase("sintetico")) {
					if(materiali[i] instanceof MaterialeSintetico){
						System.out.println(materiali[i]); // chiamata implicita al metodo toString()
						numStampati++;
					}
				}
				else if(tipo.equalsIgnoreCase("metallo")) {
					if(materiali[i] instanceof MaterialeMetallo){
						System.out.println(materiali[i]); // chiamata implicita al metodo toString()
						numStampati++;
					}		
				}
				else
					System.out.println("Errore: tipo di materiale non differenziabile!");
			} // end if
		} // end for
		return numStampati;
	}	
	
	*/
	
	
	// punto 4)
	// versione con metodi per l'accesso posizionale alle liste e uso del metodo 2)
		public DatiMateriale puntiPerMaterialeConMetodo2(int codiceMateriale, String provincia){
			DatiMateriale dati = null; 
			Materiale materiale = null;
			String tipo = ""; 
			int numPuntiPerMateriale=0;
			int numPuntiPerProvincia=0;
			
			materiale = cercaMateriale(codiceMateriale);
			if(materiale == null)
				System.out.println("Materiale non presente nel vettore");
			else{
				if(materiale instanceof MaterialeBiologico)
					tipo = "biologico";
				else if(materiale instanceof MaterialeSintetico)
						tipo = "sintetico";
				else if(materiale instanceof MaterialeMetallo)
						tipo = "metallo";
				for(int i=0; i<punti.size(); i++)
					if(punti.get(i).getProvincia().equalsIgnoreCase(provincia)){
						numPuntiPerProvincia++;
						if(punti.get(i).getTipoMaterialeRaccolto().equalsIgnoreCase(tipo))	
							numPuntiPerMateriale++;
					}
				if(numPuntiPerMateriale==0)
					System.out.println("Non sono presenti punti di raccolta per il materiale cercato nella provincia indicata");
				else{
					double percentualePunti = (double)numPuntiPerMateriale/numPuntiPerProvincia*100;
					dati = new DatiMateriale(numPuntiPerMateriale,percentualePunti);
				}
			}
			return dati;
		}
		
	// versione con metodi per l'accesso posizionale alle liste, senza uso del metodo 2)
	public DatiMateriale puntiPerMateriale(int codiceMateriale, String provincia){
		DatiMateriale dati = null; 
		String tipo=null; 
		int numPuntiPerMateriale=0;
		int numPuntiPerProvincia=0;
		
		boolean trovato=false;
		for(int i=0; i<numMateriali && !trovato; i++)
			if(materiali[i].getCodice()==codiceMateriale){
				trovato=true;
				if(materiali[i] instanceof MaterialeBiologico)
					tipo = "biologico";
				else if(materiali[i] instanceof MaterialeSintetico)
						tipo = "sintetico";
				else if(materiali[i] instanceof MaterialeMetallo)
						tipo = "metallo";
			}
		if(!trovato)
			System.out.println("Materiale non presente nel vettore");
		else{
			for(int i=0; i<punti.size(); i++)
				if(punti.get(i).getProvincia().equalsIgnoreCase(provincia)){
					numPuntiPerProvincia++;
					if(punti.get(i).getTipoMaterialeRaccolto().equalsIgnoreCase(tipo))	
						numPuntiPerMateriale++;
				}
			if(numPuntiPerMateriale==0)
				System.out.println("Non sono presenti punti di raccolta per il materiale cercato nella provincia indicata");
			else{
				double percentualePunti = (double)numPuntiPerMateriale/numPuntiPerProvincia*100;
				dati = new DatiMateriale(numPuntiPerMateriale,percentualePunti);
			}
		}
		return dati;
	}
	
	// versione con iteratore, con uso del metodo 2)	
	public DatiMateriale puntiPerMaterialeConIteratoreConMetodo2(int codiceMateriale, String provincia){
		DatiMateriale dati = null; 
		Materiale materiale = null;
		String tipo=null; 
		int numPuntiPerMateriale=0;
		int numPuntiPerProvincia=0;
			
		materiale = cercaMateriale(codiceMateriale);
		if(materiale == null)
			System.out.println("Materiale non presente nel vettore");
		else{
			if(materiale instanceof MaterialeBiologico)
				tipo = "biologico";
			else if(materiale instanceof MaterialeSintetico)
					tipo = "sintetico";
			else if(materiale instanceof MaterialeMetallo)
					tipo = "metallo";
			Iterator<Punto> i = punti.iterator();
			while(i.hasNext()) {
				Punto p = i.next();
				if(p.getProvincia().equalsIgnoreCase(provincia)){
					numPuntiPerProvincia++;
					if(p.getTipoMaterialeRaccolto().equalsIgnoreCase(tipo))	
						numPuntiPerMateriale++;
				}
			}
			if(numPuntiPerMateriale==0)
				System.out.println("Non sono presenti punti di raccolta per il materiale cercato nella provincia indicata");
			else{
				double percentualePunti = (double)numPuntiPerMateriale/numPuntiPerProvincia*100;
				dati = new DatiMateriale(numPuntiPerMateriale,percentualePunti);
			}
		}
		return dati;
	}

	// versione con iteratore, senza uso del metodo 2)
	public DatiMateriale puntiPerMaterialeConIteratore(int codiceMateriale, String provincia){
		DatiMateriale dati = null; 
		String tipo=null; 
		int numPuntiPerMateriale=0;
		int numPuntiPerProvincia=0;
			
		boolean trovato=false;
		for(int i=0; i<numMateriali && !trovato; i++)
			if(materiali[i].getCodice()==codiceMateriale){
				trovato=true;
				if(materiali[i] instanceof MaterialeBiologico)
					tipo = "biologico";
				else if(materiali[i] instanceof MaterialeSintetico)
					tipo = "sintetico";
				else if(materiali[i] instanceof MaterialeMetallo)
					tipo = "metallo";
			}
		if(!trovato)
			System.out.println("Materiale non presente nel vettore");
		else{
			Iterator<Punto> i = punti.iterator();
			while(i.hasNext()) {
				Punto p = i.next();
				if(p.getProvincia().equalsIgnoreCase(provincia)){
					numPuntiPerProvincia++;
					if(p.getTipoMaterialeRaccolto().equalsIgnoreCase(tipo))	
						numPuntiPerMateriale++;
				}
			}
			if(numPuntiPerMateriale==0)
				System.out.println("Non sono presenti punti di raccolta per il materiale cercato nella provincia indicata");
			else{
				double percentualePunti = (double)numPuntiPerMateriale/numPuntiPerProvincia*100;
				dati = new DatiMateriale(numPuntiPerMateriale,percentualePunti);
			}
		}
		return dati;
	}

}
